<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

/**
 * Class DLM_NF_Ninja_Forms_Handler
 */
class DLM_GF_Gravity_Forms_Handler {

	/**
	 *
	 */
	public function setup() {

		// register new field
		add_action( 'admin_init', array( $this, 'register_field' ) );

		// field input
		add_action( 'gform_field_input', array( $this, 'field_input' ), 10, 5 );

		// Filter the custom field
		add_filter( 'gform_field_content', array( $this, 'add_field' ), 10, 2 );

		// Set entry id in cookie
		add_action( 'gform_after_submission', array( $this, 'set_entry_id_cookie' ), 10, 2 );

		// Add download button to success message
		add_filter( 'gform_confirmation', array( $this, 'add_download_button_to_success_msg' ), 10, 3 );
	}

	/**
	 * Register our custom field
	 */
	public function register_field() {
		try {
			if ( class_exists( 'GF_Fields' ) ) {
				GF_Fields::register( new DLM_GF_DLM_Field() );
			}
		} catch ( Exception $e ) {
		}
	}

	/**
	 * Set field input
	 *
	 * @param $input
	 * @param $field
	 * @param $value
	 * @param $lead_id
	 * @param $form_id
	 *
	 * @return string
	 */
	public function field_input( $input, $field, $value, $lead_id, $form_id ) {
		if ( 'dlm_gf' === $field['type'] ) {
			$input = sprintf( "<div class='ginput_container'><p><small>%s</small></p></div>", __( "Adding this field is all you need to do, there's no other configuration required. You will not see this field on the frontend of your website.", 'dlm-gravity-forms' ) );
		}

		return $input;
	}


	/**
	 * Add custom field
	 *
	 * @param string $input
	 * @param array $field
	 *
	 * @return string
	 */
	public function add_field( $input, $field ) {

		// don't load for backend
		if ( is_admin() ) {
			return $input;
		}

		if ( 'dlm_gf' === $field['type'] ) {

			// get field object value
			$input_val = $field->get_value_submission( array() );

			// set extension variable if field value is empty
			if ( empty( $input_val ) ) {
				$input_val = '%DOWNLOAD_ID%';
			}

			$input = '';

			$input .= '<div class"ginput_container"><input type="hidden" name="input_' . $field['id'] . '" value="' . esc_attr( $input_val ) . '" /></div>';
		}

		return $input;
	}

	/**
	 * Set entry id in cookie
	 *
	 * @param array $entry
	 * @param array $form
	 */
	public function set_entry_id_cookie( $entry, $form ) {

		// Get lead index
		$lead_id = $this->get_lead_index_of_dlm_field( $form );

		// Check
		if ( null !== $lead_id ) {

			// Check if our field is in the entry array
			if ( isset( $entry[ $lead_id ] ) ) {

				// The download ID, finally
				$download_id = absint( $entry[ $lead_id ] );

				// Check download ID
				if ( $download_id > 0 ) {

					// Headers shouldn't be sent
					if ( ! headers_sent() ) {

						// Generate download hash
						$download_hash = md5( time() . mt_rand() . 'hallometad' );

						// Append download hash to download id.
						$entry[ $lead_id ] = $download_id . '|' . $download_hash;

						$update_success = GFAPI::update_entry( $entry );

						// Check for entry update error
						if ( is_wp_error( $update_success ) ) {
							wp_die( 'There was a problem granting download access. Please try again later.' );
						}

						// Set the cookie
						setcookie( DLM_GF_Constants::COOKIE, json_encode( array(
							'sub_id'        => $entry['id'],
							'form_id'       => $form['id'],
							'download_id'   => $download_id,
							'download_hash' => $download_hash
						) ), apply_filters( 'dlm_gf_cookie_expire_time', 0, $download_id ), '/' );
					}

				}
			}

		}

	}

	/**
	 * Add download button to success message if this is a form send
	 *
	 * @param string|array $confirmation
	 * @param array $form
	 * @param array $lead
	 *
	 * @return string
	 */
	public function add_download_button_to_success_msg( $confirmation, $form, $lead ) {

		// Get lead index
		$lead_id = $this->get_lead_index_of_dlm_field( $form );

		// Check
		if ( null !== $lead_id ) {

			// Check if our field is in the entry array
			if ( isset( $lead[ $lead_id ] ) ) {

				// The download ID, finally
				$download_id = absint( $lead[ $lead_id ] );

				// Get Download
				try {
					$download = download_monitor()->service( 'download_repository' )->retrieve_single( $download_id );
				} catch ( Exception $exception ) {
					return $confirmation;
				}

				if ( ! is_array( $confirmation ) ) {

					// template handler
					$template_handler = new DLM_Template_Handler();

					// load template
					ob_start();
					$template_handler->get_template_part( 'gf-download-link', '', plugin_dir_path( DLM_Gravity_Forms::get_plugin_file() ) . 'templates/', array( 'download' => $download ) );
					$confirmation .= ob_get_clean();

				}
			}

		}

		return $confirmation;
	}

	/**
	 * Get the lead index of the DLM field. The lead index is dynamic because it's generated once the dlm_gf is added to the form.
	 *
	 * @param $form
	 *
	 * @return int
	 */
	public function get_lead_index_of_dlm_field( $form ) {

		$lead_index = null;

		// Check fields
		if ( isset( $form['fields'] ) && is_array( $form['fields'] ) && count( $form ) > 0 ) {

			// Loop
			foreach ( $form['fields'] as $field ) {

				// Search for the DLM field
				if ( 'dlm_gf' === $field['type'] ) {

					$lead_index = $field['id'];

					break;
				}
			}
		}

		return $lead_index;
	}

}