<?php
/*
	Plugin Name: Download Monitor - Gravity Forms Extension
	Plugin URI: https://www.download-monitor.com/
	Description: The Gravity Forms extension for Download Monitor allows you to require users to fill out a Gravity Forms form before they gain access to a download.
	Version: 4.0.1
	Author: Never5
	Author URI: http://www.never5.com/
	License: GPL v3
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

class DLM_Gravity_Forms {

	const VERSION = '4.0.1';

	public function __construct() {

		// Check if
		if ( $this->check_dependencies() ) {
			$this->setup();
		} else {
			add_action( 'admin_notices', array( $this, 'dependency_notice' ) );
		}

	}

	/**
	 * Check for plugin dependencies
	 *
	 * @return bool
	 */
	private function check_dependencies() {
		if ( class_exists( 'WP_DLM' ) && version_compare( DLM_VERSION, '4.0.0', '>=' ) ) {
			return true;
		}

		return false;
	}

	/**
	 * Add dependency notice
	 */
	public function dependency_notice() {
		?>
        <div class="error">
            <p><?php _e( 'Download Monitor 4.0.0 or higher needs to be activated for the Download Monitor Gravity Forms Extension!', 'dlm-gravity-forms' ); ?></p>
        </div>
		<?php
	}

	/**
	 * Gravity Forms missing notice
	 */
	public function gf_missing_notice() {
		?>
        <div class="error">
            <p><?php _e( 'Gravity Forms needs to be activated for the Download Monitor Gravity Forms Extension!', 'dlm-gravity-forms' ); ?></p>
        </div>
		<?php
	}

	/**
	 * Setup the plugin
	 */
	public function setup() {

		// Download Access Manager
		$access_manager = new DLM_GF_Access_Manager();
		$access_manager->setup();

		// add unlock form the no access page
		add_action( 'dlm_no_access_after_message', array( $this, 'add_form_to_no_access_page' ) );

		// hijack the download shortcode
		add_shortcode( 'dlm_gf_form', array( $this, 'shortcode_dlm_gf_form' ) );

		// Gravity Forms Handler
		$nf_handler = new DLM_GF_Gravity_Forms_Handler();
		$nf_handler->setup();

		// Admin only classes
		if ( is_admin() ) {

			// GF metabox
			new DLM_GF_Meta_Box_Gravity_Forms();

			// check if correct GF is setup
			add_action( 'admin_init', array( $this, 'check_gravity_forms' ) );

			// add settings
			add_filter( 'download_monitor_settings', array( 'DLM_GF_Settings', 'add_settings' ) );
		}

		// Register Extension
		add_filter( 'dlm_extensions', array( $this, 'register_extension' ) );

	}

	/**
	 * Check if GF is active
	 */
	public function check_gravity_forms() {
		if ( ! is_plugin_active( 'gravityforms/gravityforms.php' ) ) {
			add_action( 'admin_notices', array( $this, 'gf_missing_notice' ) );
		}
	}

	/**
	 * Get the plugin file
	 *
	 * @static
	 *
	 * @return String
	 */
	public static function get_plugin_file() {
		return __FILE__;
	}

	/**
	 * Register this extension
	 *
	 * @param array $extensions
	 *
	 * @return array $extensions
	 */
	public function register_extension( $extensions ) {

		$extensions[] = array(
			'file'    => 'dlm-gravity-forms',
			'version' => self::VERSION,
			'name'    => 'Gravity Forms Lock'
		);

		return $extensions;
	}

	/**
	 * The dlm_nf_form callback
	 *
	 * @param array $atts
	 *
	 * @return string
	 */
	public function shortcode_dlm_gf_form( $atts ) {

		// the download_id is a required attribute
		if ( ! isset( $atts['download_id'] ) ) {
			return 'Download Monitor - Gravity Forms Error: No download_id set!';
		}

		// default content
		$content = 'Download Monitor - Gravity Forms Error: Download is not Gravity Form locked!';

		// get form ID
		$form_id = get_post_meta( $atts['download_id'], DLM_GF_Constants::META_FORM, true );

		if ( '' !== $form_id && 0 !== $form_id ) {

			if ( function_exists( 'gravity_form' ) ) {

				// default shortcode attributes
				$gf_shortcode_atts = array(
					'title'       => 'false',
					'description' => 'false',
					'ajax'        => 'false',
				);

				// check if shortcode atts are set & loop
				if ( is_array( $atts ) && count( $atts ) > 0 ) {
					foreach ( $atts as $att_key => $att_val ) {

						// check if it's a gf attribute
						if ( 0 === strpos( $att_key, 'gf_' ) ) {

							// add or overwrite
							$gf_shortcode_atts[ str_ireplace( 'gf_', '', $att_key ) ] = $att_val;
						}
					}

				}

				// set id attribute
				$gf_shortcode_atts['id'] = $form_id;

				// the GF shortcode attr string
				$gf_shortcode_attr_str = '';

				// make GF attributes string
				foreach ( $gf_shortcode_atts as $gf_key => $gf_val ) {
					$gf_shortcode_attr_str .= ' ' . $gf_key . '="' . $gf_val . '"';
				}

				$content = do_shortcode( '[gravityform' . $gf_shortcode_attr_str . ']' );

				// Set correct DOWNLOAD ID
				$content = str_replace( '%DOWNLOAD_ID%', $atts['download_id'], $content );
			}

		}

		return $content;
	}

	/**
	 * Add download to no access page
	 *
	 * @param DLM_Download $download
	 */
	public function add_form_to_no_access_page( $download ) {

		$form_id = get_post_meta( $download->get_id(), DLM_GF_Constants::META_FORM, true );
		if ( '' !== $form_id && 0 !== $form_id ) {

			if ( function_exists( 'gravity_form' ) ) {

				ob_start();
				gravity_form( $form_id, false, false );
				$form = ob_get_clean();

				$form = str_replace( '%DOWNLOAD_ID%', $download->get_id(), $form );

				// template handler
				$template_handler = new DLM_Template_Handler();

				// load template
				$template_handler->get_template_part( 'gf-unlock-form', '', plugin_dir_path( self::get_plugin_file() ) . 'templates/', array(
					'download' => $download,
					'form'     => $form
				) );
			}

		}
	}

}

require_once dirname( __FILE__ ) . '/vendor/autoload_52.php';

function __dlm_gravity_forms() {
	new DLM_Gravity_Forms();
}

add_action( 'plugins_loaded', '__dlm_gravity_forms', 11 );